/*****************************************************************
 * Copyright (c) 2016 Texas Instruments and others
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Paul Gingrich - Initial API and implementation
 *****************************************************************/
var gc = gc || {};
gc.databind = gc.databind || {};

/**
 * Callback interface for encoding and decoding packet data.
 * 
 * @interface
 */
gc.databind.IPacketCodec = function()
{
};

/**
 * Encodes data into a packet for sending.
 *    
 * @param data {Object} - The data object to encode into a packet for sending.  
 * @return {Number[]}  - The encoded raw packet data, as an array of bytes.   
 */
gc.databind.IPacketCodec.prototype.encode = function(data) {};

/**
 * Decodes packets into data objects.  One object for each packet of data.  The packet data is not framed,
 * so this method is responsible for decoding partial packets as well as multiple packets.  Partial packets
 * should not be returned, but deferred until the next decode method call with more raw data to process.  As a 
 * result, this method may return 0 or more packets of data on each invocation. 
 *    
 * @param rawData {Number[]} - The raw data received that needs to be decoded from packets into objects.  
 * @param streamingModel {gc.databind.AbstractStreamingDataModel} - The AbstractStreamingDataModel to push packet data objects through.  
 */
gc.databind.IPacketCodec.prototype.decode = function(rawData) {};

/**
 * Tests if the transport is connected.  The connected state should begin as false or unconnected until a 
 * valid packet is received.  Afterwich it should return true or false based on whether valid packets are 
 * being received or not.
 *    
 * @param rawData {Number[]} - The raw data received that needs to be decoded from packets into objects.  
 * @return {Object[]} - Array of decoded data objects, one for each packet received, if any.      
 */
gc.databind.IPacketCodec.prototype.isConnected = function(data) {};
